/* $Id: db.h 256 2008-04-21 13:54:49Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* XXX */
#include <time.h>
#include "splaytree.h"

struct sockaddr;

/* zone tree */
struct hint {
	struct splay_node tree;

	const char *zone;
	unsigned int class;

	/* local zones */
	int local;
	struct rrset *local_soa;

	/* non-local zones */
	int max_sac;
	int sac;
	struct sockaddr *sav[];
};

/* cache */
struct authority {
	struct cache c;
	struct splay_node tree;

	const char *zone;
	unsigned int class;

	int glue_count;
	int nsc;
	char *nsv[];
};
struct rrset {
	struct cache c;
	struct splay_node tree;

	const char *domain;
	unsigned int class;
	unsigned int type;
	/* rcode and zone are used only in negative cache */
	int rcode;
	const char *zone;

	int datac;			/* 0 is a negative cache */
	struct {
		char *p;
		size_t l;
	} datav[];
};
struct glue {		/* same with rrset */
	struct cache c;
	struct splay_node tree;

	const char *domain;
	unsigned int class;
	unsigned int type;
	/* XXX not used but to be the same format with rrset */
	int rcode;
	const char *zone;

	int datac;
	struct {
		char *p;
		size_t l;
	} datav[];
};

struct servcap {
	struct cache c;
	struct splay_node tree;

	unsigned int rtt;			/* in msec */
	int edns_version;			/* -1 = unsupported */
#define EDNS_UNKNOWN			-2
	const struct sockaddr *sa;		/* a bit waste... */
	struct timespec active;
};

struct hint *alloc_hint(const char *zone, unsigned int class);
int fixate_hints(int cancel);
struct hint *get_hint(const char **superzone,
    const char *zone, unsigned int class);

struct rrset *new_rrset(const char *domain,
    unsigned int type, unsigned int class, uint32_t ttl, int count);
struct rrset *get_rrset(const char *domain,
    unsigned int type, unsigned int class);
int foreach_rrset_any(struct rrset *head,
    int (*callback)(struct rrset *, void *), void *arg);
void replace_rrset(struct rrset *r);
void replace_rrset_as_glue(struct rrset *r);

struct authority *new_authority(const char *zone,
    unsigned int class, uint32_t ttl, int count);
struct authority *get_authority(const char **superzone,
    const char *zone, unsigned int class);
void replace_authority(struct authority *a);

struct glue *new_glue(const char *domain,
    unsigned int type, unsigned int class, uint32_t ttl, int count);
struct glue *get_glue(const char *domain,
    unsigned int type, unsigned int class);
void replace_glue(struct glue *a);

unsigned int get_servcap_rtt(const struct sockaddr *sa);
void add_servcap_rtt(const struct sockaddr *sa, unsigned int add);
void decay_servcap_rtt(const struct sockaddr *sa);
void start_servcap_rtt(const struct sockaddr *sa, struct timespec *start);
void stop_servcap_rtt(const struct sockaddr *sa, struct timespec *start,
    int cancel);
int get_servcap_edns_version(const struct sockaddr *sa);
void set_servcap_edns_version(const struct sockaddr *sa, int edns_version);
