/* $Id: format.h 284 2009-06-21 04:08:48Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* TYPE */
#define TYPE_A			1
#define TYPE_NS			2
#define TYPE_MD			3	/* obsolete */
#define TYPE_MF			4	/* obsolete */
#define TYPE_CNAME		5
#define TYPE_SOA		6
#define TYPE_MB			7	/* experimental */
#define TYPE_MG			8	/* experimental */
#define TYPE_MR			9	/* experimental */
#define TYPE_PTR		12
#define TYPE_MINFO		14
#define TYPE_MX			15
#define TYPE_TXT		16
#define TYPE_RP			17
#define TYPE_AFSDB		18
#define TYPE_RT			21
#define TYPE_SIG		24
#define TYPE_PX			26
#define TYPE_AAAA		28	/* RFC 3596 */
#define TYPE_NXT		30
#define TYPE_SRV		33
#define TYPE_NAPTR		35

#define PTYPE_OPT		41	/* RFC 2671 */

#define QTYPE_IXFR		251	/* RFC 1995 */
#define QTYPE_AXFR		252
#define QTYPE_MAILB		253
#define QTYPE_MAILA		254	/* obsolete */
#define QTYPE_ANY		255

/* CLASS */
#define CLASS_IN		1
#define CLASS_CH		3

#define QCLASS_ANY		255

/* OPCODE */
#define OPCODE_QUERY		0
#define OPCODE_IQUERY		1	/* obsoleted by RFC 3425 */
#define OPCODE_STATUS		2

/* RCODE and EDNS Extended RCODE */
#define RCODE_NOERROR		0
#define RCODE_FORMERR		1
#define RCODE_SERVFAIL		2
#define RCODE_NXDOMAIN		3
#define RCODE_NOTIMP		4
#define RCODE_REFUSED		5

#define RCODE_BADVERS		16	/* RFC 2671 */

/*
 * RFC 1035
 * 4.1.1. Header section format
 *
 * The header contains the following fields:
 *
 *                                     1  1  1  1  1  1
 *       0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |                      ID                       |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |QR|   Opcode  |AA|TC|RD|RA|   Z    |   RCODE   |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |                    QDCOUNT                    |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |                    ANCOUNT                    |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |                    NSCOUNT                    |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 *     |                    ARCOUNT                    |
 *     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
 */
struct header {
	uint16_t id;
	uint16_t flags;
	uint16_t qdcount;		/* query */
	uint16_t ancount;		/* answer */
	uint16_t nscount;		/* authority */
	uint16_t arcount;		/* additional */
};
#define QR_BIT			0x8000U
#define AA_BIT			0x0400U
#define TC_BIT			0x0200U
#define RD_BIT			0x0100U
#define RA_BIT			0x0080U
#define OPCODE_MASK		0x7800U
#define Z_MASK			0x0070U
#define RCODE_MASK		0x000fU
#define GET_OPCODE(flags)	(((flags) >> 11) & 0x0f)
#define GET_RCODE(flags)	((flags) & 0x0f)
#define EXT_RCODE_SHIFT		4


#define DNS_MAX_NAME_LEN	255
#define DNS_MAX_UDP_SIZE	512

#define DNS_DEFAULT_PORT	53


#define SECTION_QD	-1		/* handled separately */
#define SECTION_AN	0
#define SECTION_NS	1
#define SECTION_AR	2
#define SECTION_MAX	3

struct pktdec {
	const char *buf;
	const char *p;
	const char *end;
	const struct header *hd;

	struct {
		int version;		/* -1 = no OPT RR */
		uint8_t extrcode;
		uint16_t udpsize;
		uint16_t extflags;
	} opt;

	int countv[SECTION_MAX];	/* XXX little meaning any more */
	int count;

	/*
	 * Sorted index.
	 * - pktdec_idx is the chain of all RRs.  RRsets are uninterleaved.
	 *   RRset order and RR order in a RRset are preserved.
	 * - pktdec_sort is sorted RRsets.  It is used only to construct
	 *   pktdec_idex.
	 */
	struct pktdec_idx {
		const char *rr;
		struct pktdec_idx *next;
		int count;	/* in the first entry in an RRset, number of
				   RRs in the RRset; otherwise, 0 */
	} idxbuf[DNS_MSG_BUF_SIZE / 11], *idxv[SECTION_MAX], *idx;
	struct pktdec_sort {
		const char *rr;
		struct pktdec_idx *first_idx;
		struct pktdec_idx *last_idx;
	} sortbuf[DNS_MSG_BUF_SIZE / 13], *sortv[SECTION_MAX], *sortend;
	int last_rrset;

	/* parse RR */
	char name[DNS_MAX_NAME_LEN];
	uint16_t type;
	uint16_t class;
	uint32_t ttl;
	uint16_t rdlen;
	const char *rd;
	char rd_tmp[DNS_RDATA_BUF_SIZE];

	/* specific */
	uint32_t soa_minimum;
};

struct pktenc {
	char *buf;
	char *p;
	char *end;
	struct header *hd;

	int countv[SECTION_MAX];

	struct {
		uint16_t offset;
		uint16_t pointer;
	} namev[20];
	int namec;

	size_t peers_udpsize;
	struct {
		unsigned int optional : 1;
		unsigned int stop_encoding : 1;
	} ctl;

	struct {
		int version;
		uint16_t extflags;
		uint8_t extrcode;
	} opt;
};

#define PBUF_DATA(pb)			((char *)(pb)->header)
#define PBUF_TCP_PTR(pb)		((char *)(pb)->header - 2)

/*
 * "len" and "done" include the TCP length (additional 2 octets).
 * "size" does not include it.
 * Only net.c should refer len/done.
 */
struct pbuf {
	size_t size;			/* buffer size */
	size_t len;			/* data size */
	size_t done;			/* written/read data size */
	uint16_t space_for_tcp_length;
	struct header header[];
};


int dec_init(struct pktdec *pd, const char *buf, size_t len,
    const char *log_prefix);
int dec_question(struct pktdec *pd);
int dec_preparse(struct pktdec *pd);
void dec_from_section(struct pktdec *pd, int section);
int dec_is_end(struct pktdec *pd);
int dec_next_rrset(struct pktdec *pd);
int dec_parse_rr(struct pktdec *pd);
int dec_parse_rdata(struct pktdec *pd);
int dec_parse_soa_minimum(struct pktdec *pd);

int enc_init(struct pktenc *pe, char *buf, size_t len,
    int edns_version, size_t peers_udpsize, int is_tcp);
void enc_header(struct pktenc *pe, int rcode);
int enc_fini(struct pktenc *pe);
int enc_question(struct pktenc *pe,
    const char *domain, uint16_t type, uint16_t class);
int enc_rr(struct pktenc *pe, int section,
    const char *domain, uint16_t type, uint16_t class, uint32_t ttl,
    const char *rd, size_t rdlen);

size_t enc_make_soa(char *buf, size_t len,
    const char *mname, const char *rname,
    uint32_t serial, uint32_t refresh, uint32_t retry, uint32_t expire,
    uint32_t minimum);

const char *domparent(const char *d1, const char *d2);
const char *domcommon(const char *d1, const char *d2);
int domcmp(const char *d1, const char *d2);
int domcmp_nbr(const char *d1, const char *d2);
const char *domcpy(char *d, const char *s);
size_t domlen(const char *d);
char *domdup(const char *d);
int str2dom(char *d, size_t len, const char *str);

struct pbuf *alloc_pbuf(size_t size);
void free_pbuf(struct pbuf *pb);

uint16_t load16(const char *buf);
uint32_t load32(const char *buf);
void store16(char *buf, uint16_t val);
void store32(char *buf, uint32_t val);

const char *dom2str(const char *d);
