/* $Id: rclexer.l 341 2011-06-02 00:50:59Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

%{
#include <sys/socket.h>
#include <stdint.h>
#include <stdio.h>
#include <syslog.h>

#include "var.h"
#include "util.h"
#include "format.h"

#include "yy_proto.h"
#include "rcparser.h"

#define YY_NO_INPUT 1
#define YY_NO_UNPUT 1

static const char *rc_fn = "stdin";
static int rc_lineno = 1;
%}

/*
 * For STRING and QUOTED, take care not to accept dangerous patterns,
 * on which str2dom() overruns.
*/

/*
 * Escaped octets [RFC1035 5.1] [RFC4343 2.1].
 * We can use \\([^0-9]|[01][0-9][0-9]|2[0-4][0-9]|25[0-5]), but
 * semantic errors are less confusing than lexical ones in this case.
 */
UEO	\\[\.\\0-9]
QEO	\\(.|\n)

%%
zone			return ZONE;
hint			return HINT;
cache-size		return CACHE_SIZE;
listen-on		return LISTEN_ON;
service-at		return LISTEN_ON;		/* compat */
port			return PORT;
allow-query		return ALLOW_QUERY;
client-networks		return ALLOW_QUERY;		/* compat */
[ \t]+			; /* ignore white spaces */
#.*			; /* ignore comments */
[0-9]+			{ yylval.str = yytext; return NUMBER; }
([0-9A-Za-z_%\-\.:]|{UEO})+ {
				yylval.binstr.p = yytext;
				yylval.binstr.l = yyleng;
				return STRING;
			}
\"([^"\0\\]|{QEO})*\"	{
				int i;
				yylval.binstr.p = yytext;
				yylval.binstr.l = yyleng;
				for (i = 0; i < yyleng; i++)
					if (yytext[i] == '\n')
						rc_lineno++;
				return QUOTED;
			}
(.|\n)			{
				if (yytext[0] == '\n')
					rc_lineno++;
				return (unsigned char)yytext[0];
			}
%%

int
yywrap(void)
{
	return 1;
}

void
yyerror(const char *str)
{
	LERR("%s(%d): %s: %s", rc_fn, rc_lineno, yytext, str);
}

void
rc_set_file(FILE *fp, const char *fn)
{
#ifdef YY_FLUSH_BUFFER
	/*
	 * flex 2.5.4 has some buffering?  If yyparse() has returned on
	 * error (that is, stopped calling yylex() without seeing the EOF),
	 * some bytes may be remaining.
	 */
	YY_FLUSH_BUFFER;
#endif
	yyin = fp;
	rc_fn = fn;
	rc_lineno = 1;
}
