/* $Id: splaytree.h 209 2007-11-09 23:22:18Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef SPLAYTREE_H
#define SPLAYTREE_H

struct splay_node {
	struct splay_node *left;
	struct splay_node *right;
};

struct splay_descr {
	struct splay_node *root;
	int (*cmp)(const struct splay_node *, const struct splay_node *);
};

#define SPLAY_RANGE_STACKSIZE	16
struct splay_range {
	struct splay_node *max;
	struct splay_node *stack[SPLAY_RANGE_STACKSIZE];  /* forgetful stack */
	unsigned int top;
	unsigned int bottom;
};

struct splay_node *splay_find(struct splay_descr *d, struct splay_node *s);
int splay_find_leq(struct splay_descr *d, struct splay_node *s);
struct splay_node *splay_replace(struct splay_descr *d, struct splay_node *n);
struct splay_node *splay_remove(struct splay_descr *d, struct splay_node *s);
struct splay_node *splay_range_start(struct splay_descr *d,
    struct splay_range *range, struct splay_node *min, struct splay_node *max);
struct splay_node *splay_range_next(struct splay_descr *d,
    struct splay_range *range);

#endif /* !SPLAYTREE_H */
