/* $Id: stats.h 320 2010-01-14 05:52:15Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

struct stats {
	unsigned int d_query;
	unsigned int d_success;
	unsigned int d_nodata;
	unsigned int d_nxdomain;
	unsigned int d_servfail;

	unsigned int obj_pbuf_alloc;
	unsigned int obj_pbuf_fail;
	unsigned int obj_pbuf;
	unsigned int obj_pbuf_max;
	unsigned int obj_udpsock_alloc;
	unsigned int obj_udpsock_fail;
	unsigned int obj_udpsock;
	unsigned int obj_udpsock_max;
	unsigned int obj_tcpsock_alloc;
	unsigned int obj_tcpsock_fail;
	unsigned int obj_tcpsock;
	unsigned int obj_tcpsock_max;
	unsigned int obj_qtask_alloc;
	unsigned int obj_qtask_fail;
	unsigned int obj_qtask;
	unsigned int obj_qtask_max;

	unsigned int cache_expired;
	unsigned int cache_replaced;
	unsigned int cache_reclaimed;
	unsigned int cache_wasted;
};

/* downstream (communication with stub resolvers) */
#define STAT_D_QUERY(name, type, class)		(stats.d_query++)
#define STAT_D_RESPONSE(name, type, class, rcode, have_answer)	do {	\
		switch (rcode) {					\
		case RCODE_NOERROR:					\
			if (have_answer)				\
				stats.d_success++;			\
			else						\
				stats.d_nodata++;			\
			break;						\
		case RCODE_NXDOMAIN:					\
			stats.d_nxdomain++;				\
			break;						\
		case RCODE_SERVFAIL:					\
			stats.d_servfail++;				\
			break;						\
		}							\
	} while (0 /* CONSTCOND */)

/* upstream (communication with authoritative servers) */

/* memory allocation */
#define STAT_OBJ_PBUF_REQ()			((void)0)
#define STAT_OBJ_PBUF_ALLOC()			do {			\
		stats.obj_pbuf_alloc++;					\
		if (++stats.obj_pbuf > stats.obj_pbuf_max)		\
			stats.obj_pbuf_max = stats.obj_pbuf;		\
	} while (0 /* CONSTCOND */)
#define STAT_OBJ_PBUF_FAIL()			(stats.obj_pbuf_fail++)
#define STAT_OBJ_PBUF_FREE()			(stats.obj_pbuf--)
#define STAT_OBJ_UDPSOCK_REQ()			((void)0)
#define STAT_OBJ_UDPSOCK_ALLOC()		do {			\
		stats.obj_udpsock_alloc++;				\
		if (++stats.obj_udpsock > stats.obj_udpsock_max)	\
			stats.obj_udpsock_max = stats.obj_udpsock;	\
	} while (0 /* CONSTCOND */)
#define STAT_OBJ_UDPSOCK_FAIL()			(stats.obj_udpsock_fail++)
#define STAT_OBJ_UDPSOCK_FREE()			(stats.obj_udpsock--)
#define STAT_OBJ_TCPSOCK_REQ()			((void)0)
#define STAT_OBJ_TCPSOCK_ALLOC()		do {			\
		stats.obj_tcpsock_alloc++;				\
		if (++stats.obj_tcpsock > stats.obj_tcpsock_max)	\
			stats.obj_tcpsock_max = stats.obj_tcpsock;	\
	} while (0 /* CONSTCOND */)
#define STAT_OBJ_TCPSOCK_FAIL()			(stats.obj_tcpsock_fail++)
#define STAT_OBJ_TCPSOCK_FREE()			(stats.obj_tcpsock--)
#define STAT_OBJ_QTASK_REQ()			((void)0)
#define STAT_OBJ_QTASK_ALLOC()			do {			\
		stats.obj_qtask_alloc++;				\
		if (++stats.obj_qtask > stats.obj_qtask_max)		\
			stats.obj_qtask_max = stats.obj_qtask;		\
	} while (0 /* CONSTCOND */)
#define STAT_OBJ_QTASK_FAIL()			(stats.obj_qtask_fail++)
#define STAT_OBJ_QTASK_FREE()			(stats.obj_qtask--)

/* cache */
#define STAT_CACHE_EXPIRED()			(stats.cache_expired++)
#define STAT_CACHE_REPLACED()			(stats.cache_replaced++)
#define STAT_CACHE_RECLAIMED()			(stats.cache_reclaimed++)
#define STAT_CACHE_WASTED()			(stats.cache_wasted++)

void init_stats(void);
void info_stats(void);

extern struct stats stats;
