/* $Id: util.h 233 2007-12-26 14:58:08Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define LERR(...)		logmsg(LOG_ERR, __VA_ARGS__)
#define LWARN(...)		logmsg(LOG_WARNING, __VA_ARGS__)
#define LNOTICE(...)		logmsg(LOG_NOTICE, __VA_ARGS__)
#define LINFO(...)		logmsg(LOG_INFO, __VA_ARGS__)
#define LDEBUG(...)		logmsg(LOG_DEBUG, __VA_ARGS__)
#ifdef DEBUG
# define LDEBUG2(...)		logmsg2(LOG_DEBUG, __VA_ARGS__)
# define LDUMP2(pkt, len)	dump2(pkt, len)
#else
# define LDEBUG2(...)		(void)0
# define LDUMP2(pkt, len)	(void)0
#endif
#ifdef DEBUG_PACKET
# define LDEBUG2_PKT(...)	logmsg2(LOG_DEBUG, __VA_ARGS__)
# define LDUMP2_PKT(pkt, len)	dump2(pkt, len)
#else
# define LDEBUG2_PKT(...)	(void)0
# define LDUMP2_PKT(pkt, len)	(void)0
#endif

#define lengthof(array)		(sizeof(array) / sizeof((array)[0]))
#define endof(array)		((array) + lengthof(array))
/* Unconst with type check. */
#define UNCONST(type, ptr)						\
	((void)((ptr) < (const type)(ptr)), (type)(uintptr_t)(ptr))
#define STRINGIFY(str)		#str
#define NUM2STR(num)		STRINGIFY(num)
/* Convert a member to the wrapping struct (with type and const check). */
#define MBR2STRUCT(ptr, s, m)						\
	((void)((ptr) < &((s *)0)->m),					\
	 (void)(s *)(ptr),		/* XXX representable? */	\
	 (s *)(uintptr_t)((const char *)(ptr) - offsetof(s, m)))
/* Compare 16-bit values in unsigned ints. */
/* #define CMP16(x, y)		((int)(x) - (int)(y)) */
#define CMP16(x, y)		((x) > (y) ? 1 : (x) < (y) ? -1 : 0)

int run_as(const char *user);
int make_pidfile(void);
int write_pidfile(void);
void cleanup_pidfile(void);

void init_random(void);
unsigned int random_stat(unsigned int n);
unsigned int random_secure(unsigned int n);

#ifdef LOG_PERROR
#define logmsg syslog
#else
void logmsg_stderr(int enable);
void logmsg(int priority, const char *fmt, ...);
#endif

#ifdef DEBUG
void logmsg2(int priority, const char *fmt, ...);
void dump2(const char *ptr, size_t len);
#endif
