/* $Id: var.h 341 2011-06-02 00:50:59Z ken $ */
/*
 * Copyright (c) 2006 KAMADA Ken'ichi.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* Minimum (and the default) cache size. */
/* Currently, cache MUST be at least 128 times larger than DNS_MSG_BUF_SIZE. */
#define MIN_CACHE_SIZE			(1UL * 1024 * 1024)

/* Maximum UDP payload size when EDNS is used. */
#define DNS_MAX_EDNS_UDP_SIZE		1280
/* Message buffer size for composing and parsing. */
#define DNS_MSG_BUF_SIZE		4096
/* Buffer size for RDATA that includes domain-name(s). */
/* NAPTR is 1028.  SIG and NXT can be larger but have no upper bound. */
#define DNS_RDATA_BUF_SIZE		1028

/* Must be reasonably small to avoid (now + ttl) overflow. */
/* Also, no more than UINT32_MAX / 2. */
#define DNS_MAX_TTL			(86400UL * 14)
/* Minimum negative cache TTL. */
#define DNS_NEGCACHE_MIN_TTL		900UL
/* Maximum negative cache TTL. */
#define DNS_NEGCACHE_MAX_TTL		86400UL
/* Negative cache TTL when no SOA is returned. */
#define DNS_NEGCACHE_TTL_WITHOUT_SOA	300UL

/* SOA TTL for locally served "empty" zones. */
#define LOCALZONE_SOA_TTL		10800UL

/* Max length of CNAME chain. */
#define MAX_ALIAS			8

#define TIMEOUT_TRY_NEXT	1	/* timeout to try next server */
#define TIMEOUT_FINAL		3
#define TIMEOUT_PENDING		10	/* dummy; waiting other queries */

/* Max loop count. */
#define MAX_LOOP_COUNT			50
/* Max number of TCP clients. */
#define MAX_TCP_CLIENTS			10
/* Trying a lot of servers usually do more harm than good. */
#define MAX_CONTACT			4

/* Range of ephemeral port numbers. */
#define EPHEMERAL_PORT_MIN		49152
#define EPHEMERAL_PORT_MAX		65535

struct splay_descr;
struct listen_on;
struct allow_query;

struct option {
	const char *bind_port;
	int debug_level;
	int foreground;
	const char *user;
	const char *ctl_file;
};

struct conf {
	size_t cache_size;
	struct splay_descr *hint;
	struct listen_on *listen_on;
	struct allow_query *allow_query;
};

extern struct option option;
extern struct conf conf_load, conf_run;
